@@ -1,4 +1,4 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,13 +19,27 @@
 package org.apache.felix.bundlerepository.impl;
 
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
-import org.apache.felix.bundlerepository.*;
+import org.apache.felix.bundlerepository.Capability;
+import org.apache.felix.bundlerepository.Reason;
+import org.apache.felix.bundlerepository.Repository;
+import org.apache.felix.bundlerepository.Requirement;
 import org.apache.felix.bundlerepository.Resolver;
-import org.apache.felix.bundlerepository.impl.ResourceImpl;
+import org.apache.felix.bundlerepository.Resource;
 import org.apache.felix.utils.log.Logger;
-import org.osgi.framework.*;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleContext;
+import org.osgi.framework.BundleException;
+import org.osgi.framework.Constants;
+import org.osgi.framework.Version;
 
 public class ResolverImpl implements Resolver
 {
@@ -446,24 +460,41 @@ public class ResolverImpl implements Resolver
                 {
                     // If there is no best version or if the current
                     // resource's version is lower, then select it.
-                    if ((bestVersion == null) || (bestVersion.compareTo(v) < 0))
+                    if ((bestVersion == null) || (bestVersion.compareTo((Version) v) < 0))
                     {
                         best = current;
                         bestLocal = isCurrentLocal;
                         bestVersion = (Version) v;
                     }
                     // If the current resource version is equal to the
-                    // best, then select the one with the greatest
-                    // number of capabilities.
-                    else if ((bestVersion != null) && (bestVersion.compareTo(v) == 0)
-                            && (best.getResource().getCapabilities().length
-                                < current.getResource().getCapabilities().length))
+                    // best
+                    else if ((bestVersion != null) && (bestVersion.compareTo((Version) v) == 0))
                     {
-                        best = current;
-                        bestLocal = isCurrentLocal;
-                        bestVersion = (Version) v;
+                        // If the symbolic name is the same, use the highest
+                        // bundle version.
+                        if ((best.getResource().getSymbolicName() != null)
+                            && best.getResource().getSymbolicName().equals(
+                                current.getResource().getSymbolicName()))
+                        {
+                            if (best.getResource().getVersion().compareTo(
+                                current.getResource().getVersion()) < 0)
+                            {
+                                best = current;
+                                bestLocal = isCurrentLocal;
+                                bestVersion = (Version) v;
+                            }
+                        }
+                        // Otherwise select the one with the greatest
+                        // number of capabilities.
+                        else if (best.getResource().getCapabilities().length
+                            < current.getResource().getCapabilities().length)
+                        {
+                            best = current;
+                            bestLocal = isCurrentLocal;
+                            bestVersion = (Version) v;
+                        }
                     }
-                }   
+                }
             }
         }
 
@@ -559,7 +590,7 @@ public class ResolverImpl implements Resolver
                             doStartBundle = true;
                             localResource.getBundle().stop();
                         }
-                        
+
                         localResource.getBundle().update(FileUtil.openURL(new URL(deployResources[i].getURI())));
 
                         // If necessary, save the updated bundle to be
@@ -567,7 +598,7 @@ public class ResolverImpl implements Resolver
                         if (doStartBundle)
                         {
                             Bundle bundle = localResource.getBundle();
-                            if (!isFragmentBundle(bundle)) 
+                            if (!isFragmentBundle(bundle))
                             {
                                 startList.add(bundle);
                             }
@@ -605,9 +636,9 @@ public class ResolverImpl implements Resolver
                         // started later.
                         if ((flags & START) != 0)
                         {
-                            if (!isFragmentBundle(bundle)) 
+                            if (!isFragmentBundle(bundle))
                             {
-                                startList.add(bundle);    
+                                startList.add(bundle);
                             }
                         }
                     }
@@ -616,7 +647,7 @@ public class ResolverImpl implements Resolver
                 {
                     m_logger.log(
                         Logger.LOG_ERROR,
-                        "Resolver: Install error - " + deployResources[i].getSymbolicName(), 
+                        "Resolver: Install error - " + deployResources[i].getSymbolicName(),
                         ex);
                     return;
                 }
@@ -641,15 +672,15 @@ public class ResolverImpl implements Resolver
 
     /**
      * Determines if the given bundle is a fragement bundle.
-     * 
+     *
      * @param bundle bundle to check
      * @return flag indicating if the given bundle is a fragement
      */
-    private boolean isFragmentBundle(Bundle bundle) 
+    private boolean isFragmentBundle(Bundle bundle)
     {
         return bundle.getHeaders().get(Constants.FRAGMENT_HOST) != null;
     }
-    
+
     // TODO: OBR - Think about this again and make sure that deployment ordering
     // won't impact it...we need to update the local state too.
     private LocalResourceImpl findUpdatableLocalResource(Resource resource)
@@ -772,7 +803,7 @@ public class ResolverImpl implements Resolver
 
     public static String getBundleName(Bundle bundle)
     {
-        String name = (String) bundle.getHeaders().get(Constants.BUNDLE_NAME);
+        String name = bundle.getHeaders().get(Constants.BUNDLE_NAME);
         return (name == null)
             ? "Bundle " + Long.toString(bundle.getBundleId())
             : name;
